
#ifdef FIX_KVNO
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

#include <v5gen-rewrite.h>
#include <v5gen.h>


/******************************************************************/
/* Used to change a null kvno which defaults to 0 to "fixkvno"    */
/* needed since W2K is not setting the knvo, and OpenAFS 1.2.8    */
/* does not know how to handle a kvno=0. 						  */
/* Also the KeyFile has the matching key listed as kvno=1         */
/* I will have to fix this on my AFS cell to use kvno=0           */
/* We will use the rxkad v5* asn1 routines to rebuild the ticket. */


int
fix_kvno( int fixkvno, char * oldtkt, size_t oldlen, 
		  char** pnewtkt, size_t * pnewlen)
{

#define NEWTKTMAX 4096
	char * newtkt = NULL;
    size_t newtktlen = 0;


    Ticket t5; /* Must free */
    size_t siz;
    int code;

    memset(&t5, 0, sizeof(t5));
	newtkt = malloc(NEWTKTMAX);
	if (newtkt == NULL) {
		code = -1 /*ENOMEM*/;
		goto cleanup;
	}
	memset(newtkt,0,NEWTKTMAX);

    code = decode_Ticket(oldtkt, 
                      oldlen,
                     &t5, &siz);
    if (code != 0) {
		fprintf(stderr,"Failed %d %d\n",code, __LINE__);
		goto cleanup;
	}

    if (t5.enc_part.kvno == NULL ) {
        t5.enc_part.kvno = malloc(sizeof(int));
    }
	*t5.enc_part.kvno = fixkvno;

    code = encode_Ticket(newtkt+NEWTKTMAX-1, 
			NEWTKTMAX, &t5, &newtktlen);
    if (code != 0) {
			fprintf(stderr,"Failed %d %d\n",code, __LINE__);
			goto cleanup;
    }

	*pnewtkt = newtkt+NEWTKTMAX-newtktlen;
	*pnewlen = newtktlen;

cleanup:
	free_Ticket(&t5);

	return code;
}
#endif
